@@ -75,6 +75,7 @@ class Redis
       "eval"             => [ :eval_style ],
       "evalsha"          => [ :eval_style ],
       "get"              => [ :first ],
+      "getex"            => [ :first ],
       "getbit"           => [ :first ],
       "getrange"         => [ :first ],
       "getset"           => [ :first ],
@@ -151,6 +152,7 @@ class Redis
       "sinterstore"      => [ :all ],
       "sismember"        => [ :first ],
       "smembers"         => [ :first ],
+      "smismember"       => [ :first ],
       "smove"            => [ :exclude_last ],
       "sort"             => [ :sort  ],
       "spop"             => [ :first ],
@@ -242,7 +244,7 @@ class Redis
 
     def initialize(namespace, options = {})
       @namespace = namespace
-      @redis = options[:redis] || Redis.current
+      @redis = options[:redis] || Redis.new
       @warning = !!options.fetch(:warning) do
                    !ENV['REDIS_NAMESPACE_QUIET']
                  end
@@ -261,7 +263,7 @@ class Redis
     end
 
     def client
-      warn("The client method is deprecated as of redis-rb 4.0.0, please use the new _client" +
+      warn("The client method is deprecated as of redis-rb 4.0.0, please use the new _client " +
             "method instead. Support for the old method will be removed in redis-namespace 2.0.") if @has_new_client_method && deprecations?
       _client
     end
@@ -307,7 +309,7 @@ class Redis
                                    :redis => @redis)
       end
 
-      @namespace
+      @namespace.respond_to?(:call) ? @namespace.call : @namespace
     end
 
     def full_namespace
@@ -315,7 +317,7 @@ class Redis
     end
 
     def connection
-      @redis.connection.tap { |info| info[:namespace] = @namespace }
+      @redis.connection.tap { |info| info[:namespace] = namespace }
     end
 
     def exec
@@ -430,6 +432,7 @@ class Redis
           args = add_namespace(args)
         end
       when :alternate
+        args = args.flatten
         args.each_with_index { |a, i| args[i] = add_namespace(a) if i.even? }
       when :sort
         args[0] = add_namespace(args[0]) if args[0]
@@ -492,6 +495,12 @@ class Redis
     end
     ruby2_keywords(:call_with_namespace) if respond_to?(:ruby2_keywords, true)
 
+  protected
+
+    def redis=(redis)
+      @redis = redis
+    end
+
   private
 
     if Hash.respond_to?(:ruby2_keywords_hash)
@@ -520,18 +529,19 @@ class Redis
     end
 
     def namespaced_block(command, &block)
-      redis.send(command) do |r|
-        begin
-          original, @redis = @redis, r
-          yield self
-        ensure
-          @redis = original
+      if block.arity == 0
+        redis.send(command, &block)
+      else
+        redis.send(command) do |r|
+          copy = dup
+          copy.redis = r
+          yield copy
         end
       end
     end
 
     def add_namespace(key)
-      return key unless key && @namespace
+      return key unless key && namespace
 
       case key
       when Array
@@ -540,12 +550,12 @@ class Redis
         key.keys.each {|k| key[add_namespace(k)] = key.delete(k)}
         key
       else
-        "#{@namespace}:#{key}"
+        "#{namespace}:#{key}"
       end
     end
 
     def rem_namespace(key)
-      return key unless key && @namespace
+      return key unless key && namespace
 
       case key
       when Array
@@ -557,7 +567,7 @@ class Redis
           key.each { |k| yielder.yield rem_namespace(k) }
         end
       else
-        key.to_s.sub(/\A#{@namespace}:/, '')
+        key.to_s.sub(/\A#{namespace}:/, '')
       end
     end
 
