@@ -3,28 +3,17 @@
 require File.dirname(__FILE__) + '/spec_helper'
 
 describe "redis" do
-  @redis_version = Gem::Version.new(Redis.current.info["redis_version"])
+  @redis_version = Gem::Version.new(Redis.new.info["redis_version"])
   let(:redis_client) { @redis.respond_to?(:_client) ? @redis._client : @redis.client}
 
-  before(:all) do
+  before(:each) do
     # use database 15 for testing so we dont accidentally step on your real data
     @redis = Redis.new :db => 15
-  end
-
-  before(:each) do
+    @redis.flushdb
     @namespaced = Redis::Namespace.new(:ns, :redis => @redis)
-    @redis.flushdb
     @redis.set('foo', 'bar')
   end
 
-  after(:each) do
-    @redis.flushdb
-  end
-
-  after(:all) do
-    @redis.quit
-  end
-
   # redis-rb 3.3.4+
   it "should inject :namespace into connection info" do
     info = @redis.connection.merge(:namespace => :ns)
@@ -52,6 +41,17 @@ describe "redis" do
     expect(@namespaced.type('counter')).to eq('string')
   end
 
+  it "should work with Proc namespaces" do
+    namespace = Proc.new { :dynamic_ns }
+    namespaced = Redis::Namespace.new(namespace, redis: @redis)
+
+    expect(namespaced.get('foo')).to eq(nil)
+    namespaced.set('foo', 'chris')
+    expect(namespaced.get('foo')).to eq('chris')
+    @redis.set('foo', 'bob')
+    expect(@redis.get('foo')).to eq('bob')
+  end
+
   context 'when sending capital commands (issue 68)' do
     it 'should be able to use a namespace' do
       @namespaced.send('SET', 'fubar', 'quux')
@@ -106,6 +106,13 @@ describe "redis" do
     expect(@namespaced.lrange('bar',0,-1)).to eq(['bar'])
   end
 
+  it "should be able to use a namespace with getex" do
+    expect(@namespaced.set('mykey', 'Hello')).to eq('OK')
+    expect(@namespaced.getex('mykey', ex: 50)).to eq('Hello')
+    expect(@namespaced.get('mykey')).to eq('Hello')
+    expect(@namespaced.ttl('mykey')).to eq(50)
+  end
+
   it 'should be able to use a namespace with getbit' do
     @namespaced.set('foo','bar')
     expect(@namespaced.getbit('foo',1)).to eq(1)
@@ -181,15 +188,22 @@ describe "redis" do
     @namespaced.mset('foo', '1000', 'bar', '2000')
     expect(@namespaced.mapped_mget('foo', 'bar')).to eq({ 'foo' => '1000', 'bar' => '2000' })
     expect(@namespaced.mapped_mget('foo', 'baz', 'bar')).to eq({ 'foo' => '1000', 'bar' => '2000', 'baz' => nil})
+
     @namespaced.mapped_mset('foo' => '3000', 'bar' => '5000')
     expect(@namespaced.mapped_mget('foo', 'bar')).to eq({ 'foo' => '3000', 'bar' => '5000' })
     expect(@namespaced.mapped_mget('foo', 'baz', 'bar')).to eq({ 'foo' => '3000', 'bar' => '5000', 'baz' => nil})
+
+    @namespaced.mset(['foo', '4000'], ['baz', '6000'])
+    expect(@namespaced.mapped_mget('foo', 'bar', 'baz')).to eq({ 'foo' => '4000', 'bar' => '5000', 'baz' => '6000' })
   end
 
   it "should be able to use a namespace with msetnx" do
     @namespaced.msetnx('foo', '1000', 'bar', '2000')
     expect(@namespaced.mapped_mget('foo', 'bar')).to eq({ 'foo' => '1000', 'bar' => '2000' })
     expect(@namespaced.mapped_mget('foo', 'baz', 'bar')).to eq({ 'foo' => '1000', 'bar' => '2000', 'baz' => nil})
+
+    @namespaced.msetnx(['baz', '4000'])
+    expect(@namespaced.mapped_mget('foo', 'baz', 'bar')).to eq({ 'foo' => '1000', 'bar' => '2000', 'baz' => '4000'})
   end
 
   it "should be able to use a namespace with mapped_msetnx" do
@@ -398,6 +412,27 @@ describe "redis" do
     expect(result).to eq(["bar", "value"])
   end
 
+  it "is thread safe for multi blocks" do
+    mon = Monitor.new
+    entered = false
+    entered_cond = mon.new_cond
+
+    thread = Thread.new do
+      mon.synchronize do
+        entered_cond.wait_until { entered }
+        @namespaced.multi
+      end
+    end
+
+    @namespaced.multi do |transaction|
+      entered = true
+      mon.synchronize { entered_cond.signal }
+      thread.join(0.1)
+      transaction.get("foo")
+    end
+    thread.join
+  end
+
   it "should add namespace to strlen" do
     @namespaced.set("mykey", "123456")
     expect(@namespaced.strlen("mykey")).to eq(6)
@@ -677,7 +712,7 @@ describe "redis" do
               expect(result).to match_array(namespaced_keys)
             end
           end
-        end if Redis.current.respond_to?(:scan)
+        end if Redis.new.respond_to?(:scan)
 
         context '#scan_each' do
           context 'when :match supplied' do
@@ -710,7 +745,7 @@ describe "redis" do
               end
             end
           end
-        end if Redis.current.respond_to?(:scan_each)
+        end if Redis.new.respond_to?(:scan_each)
       end
 
       context 'hash scan methods' do
@@ -738,7 +773,7 @@ describe "redis" do
               expect(results).to match_array(@redis.hgetall('ns:hsh').to_a)
             end
           end
-        end if Redis.current.respond_to?(:hscan)
+        end if Redis.new.respond_to?(:hscan)
 
         context '#hscan_each' do
           context 'when :match supplied' do
@@ -771,7 +806,7 @@ describe "redis" do
               end
             end
           end
-        end if Redis.current.respond_to?(:hscan_each)
+        end if Redis.new.respond_to?(:hscan_each)
       end
 
       context 'set scan methods' do
@@ -799,7 +834,7 @@ describe "redis" do
               expect(results).to match_array(set)
             end
           end
-        end if Redis.current.respond_to?(:sscan)
+        end if Redis.new.respond_to?(:sscan)
 
         context '#sscan_each' do
           context 'when :match supplied' do
@@ -832,7 +867,7 @@ describe "redis" do
               end
             end
           end
-        end if Redis.current.respond_to?(:sscan_each)
+        end if Redis.new.respond_to?(:sscan_each)
       end
 
       context 'zset scan methods' do
@@ -862,7 +897,7 @@ describe "redis" do
               expect(results).to match_array(hash.to_a)
             end
           end
-        end if Redis.current.respond_to?(:zscan)
+        end if Redis.new.respond_to?(:zscan)
 
         context '#zscan_each' do
           context 'when :match supplied' do
@@ -895,7 +930,7 @@ describe "redis" do
               end
             end
           end
-        end if Redis.current.respond_to?(:zscan_each)
+        end if Redis.new.respond_to?(:zscan_each)
       end
     end
   end
